#include "hashint.h"
#include "hashtable.h"
#include "listint.h"
#include "list.h"
#include "keyint.h"

//////////////////////////////////////////////////////////////////////////////
// CHashInt Class
//

CHashInt::CHashInt()
{
  m_pCHashTable = NULL;
}

CHashInt::~CHashInt()
{
  Uninit();
}

BOOL CHashInt::Init(INT_32 uiBuckets/* = 127*/)
{
  ASSERT( m_pCHashTable == NULL );

  m_pCHashTable = new CHashTable<CList<VOID*, CKeyInt>, VOID*, CKeyInt>;
  ASSERT( m_pCHashTable != NULL );

  return ((CHashTable<CList<VOID*, CKeyInt>, VOID*, CKeyInt>*)m_pCHashTable)->Init(uiBuckets);
}

VOID CHashInt::Uninit()
{
  if ( m_pCHashTable != NULL )
    {
      delete m_pCHashTable;
      m_pCHashTable = NULL;
    }
}

VOID *CHashInt::Insert(VOID *pvData, const INT_32 iKey)
{
  CKeyInt key;

  ASSERT( m_pCHashTable != NULL );

  key.Set(iKey);

  return ((CHashTable<CList<VOID*, CKeyInt>, VOID*, CKeyInt>*)m_pCHashTable)->Insert(pvData, key);
}

VOID *CHashInt::Peek(const INT_32 iKey)
{
  CKeyInt key;

  ASSERT( m_pCHashTable != NULL );

  key.Set(iKey);

  return ((CHashTable<CList<VOID*, CKeyInt>, VOID*, CKeyInt>*)m_pCHashTable)->Peek(key);
}

VOID *CHashInt::Remove(const INT_32 iKey)
{
  CKeyInt key;

  ASSERT( m_pCHashTable != NULL );

  key.Set(iKey);

  return ((CHashTable<CList<VOID*, CKeyInt>, VOID*, CKeyInt>*)m_pCHashTable)->Remove(key);
}

VOID *CHashInt::RemoveFirst()
{
  ASSERT( m_pCHashTable != NULL );

  return ((CHashTable<CList<VOID*, CKeyInt>, VOID*, CKeyInt>*)m_pCHashTable)->RemoveFirst();
}

VOID CHashInt::Display()
{
  ASSERT( m_pCHashTable != NULL );

  ((CHashTable<CList<VOID*, CKeyInt>, VOID*, CKeyInt>*)m_pCHashTable)->Display();
}

//////////////////////////////////////////////////////////////////////////////
// CListInt Class
//

CListInt::CListInt()
{
  m_pCList = NULL;
}

CListInt::~CListInt()
{
  Uninit();
}

BOOL CListInt::Init()
{
  ASSERT( m_pCList == NULL );

  m_pCList = new CList<VOID*, CKeyInt>;
  ASSERT( m_pCList != NULL );

  return ((CList<VOID*, CKeyInt>*)m_pCList)->Init();
}

VOID CListInt::Uninit()
{
  if ( m_pCList != NULL )
    {
      delete m_pCList;
      m_pCList = NULL;
    }
}

VOID *CListInt::InsertForward(VOID *pvData, INT_32 iKey)
{
  CKeyInt key;

  ASSERT( m_pCList != NULL );

  key.Set(iKey);

  return ((CList<VOID*, CKeyInt>*)m_pCList)->InsertForward(pvData, key);
}

VOID *CListInt::InsertBackward(VOID *pvData, INT_32 iKey)
{
  CKeyInt key;

  ASSERT( m_pCList != NULL );

  key.Set(iKey);

  return ((CList<VOID*, CKeyInt>*)m_pCList)->InsertBackward(pvData, key);
}

VOID *CListInt::InsertFirst(VOID *pvData, INT_32 iKey)
{
  CKeyInt key;

  ASSERT( m_pCList != NULL );

  key.Set(iKey);

  return ((CList<VOID*, CKeyInt>*)m_pCList)->InsertFirst(pvData, key);
}

VOID *CListInt::InsertLast(VOID *pvData, INT_32 iKey)
{
  CKeyInt key;

  ASSERT( m_pCList != NULL );

  key.Set(iKey);

  return ((CList<VOID*, CKeyInt>*)m_pCList)->InsertLast(pvData, key);
}

VOID *CListInt::RemoveForward(INT_32 iKey)
{
  CKeyInt key;

  ASSERT( m_pCList != NULL );

  key.Set(iKey);

  return ((CList<VOID*, CKeyInt>*)m_pCList)->RemoveForward(key);
}

VOID *CListInt::RemoveBackward(INT_32 iKey)
{
  CKeyInt key;

  ASSERT( m_pCList != NULL );

  key.Set(iKey);

  return ((CList<VOID*, CKeyInt>*)m_pCList)->RemoveBackward(key);
}

VOID *CListInt::RemoveFirst(INT_32 *piKey/* = NULL*/)
{
  VOID *pvData;

  ASSERT( m_pCList != NULL );

  if ( piKey == NULL )
    {
      pvData = ((CList<VOID*, CKeyInt>*)m_pCList)->RemoveFirst(NULL);
    }
  else
    {
      CKeyInt key;
      key.SetNull();
      pvData = ((CList<VOID*, CKeyInt>*)m_pCList)->RemoveFirst(&key);
      *piKey = key.Get();
    }

  return pvData;
}

VOID *CListInt::RemoveLast(INT_32 *piKey/* = NULL*/)
{
  VOID *pvData;

  ASSERT( m_pCList != NULL );

  if ( piKey == NULL )
    {
      pvData = ((CList<VOID*, CKeyInt>*)m_pCList)->RemoveLast(NULL);
    }
  else
    {
      CKeyInt key;
      key.SetNull();
      pvData = ((CList<VOID*, CKeyInt>*)m_pCList)->RemoveLast(&key);
      *piKey = key.Get();
    }

  return pvData;
}

VOID CListInt::RemoveAll()
{
  ASSERT( m_pCList != NULL );

  ((CList<VOID*, CKeyInt>*)m_pCList)->RemoveAll();
}

VOID *CListInt::PeekForward(INT_32 iKey) const
{
  CKeyInt key;

  ASSERT( m_pCList != NULL );

  key.Set(iKey);

  return ((CList<VOID*, CKeyInt>*)m_pCList)->PeekForward(key);
}

VOID *CListInt::PeekBackward(INT_32 iKey) const
{
  CKeyInt key;

  ASSERT( m_pCList != NULL );

  key.Set(iKey);

  return ((CList<VOID*, CKeyInt>*)m_pCList)->PeekBackward(key);
}

VOID *CListInt::PeekFirst(INT_32 *piKey/* = NULL*/) const
{
  VOID *pvData;

  ASSERT( m_pCList != NULL );

  if ( piKey == NULL )
    {
      pvData = ((CList<VOID*, CKeyInt>*)m_pCList)->PeekFirst(NULL);
    }
  else
    {
      CKeyInt key;
      key.SetNull();
      pvData = ((CList<VOID*, CKeyInt>*)m_pCList)->PeekFirst(&key);
      *piKey = key.Get();
    }

  return pvData;
}

VOID *CListInt::PeekLast(INT_32 *piKey/* = NULL*/) const
{
  VOID *pvData;

  ASSERT( m_pCList != NULL );

  if ( piKey == NULL )
    {
      pvData = ((CList<VOID*, CKeyInt>*)m_pCList)->PeekLast(NULL);
    }
  else
    {
      CKeyInt key;
      key.SetNull();
      pvData = ((CList<VOID*, CKeyInt>*)m_pCList)->PeekLast(&key);
      *piKey = key.Get();
    }

  return pvData;
}

VOID CListInt::Display()
{
  ASSERT( m_pCList != NULL );

  ((CList<VOID*, CKeyInt>*)m_pCList)->Display();
}

#ifdef SORT_ENABLED

BOOL CListInt::Sort()
{
  ASSERT( m_pCList != NULL );

  return ((CList<VOID*, CKeyInt>*)m_pCList)->Sort();
}

#endif // SORT_ENABLED

UINT_32 CListInt::GetLength()
{
  ASSERT( m_pCList != NULL );

  return ((CList<VOID*, CKeyInt>*)m_pCList)->GetLength();
}

//////////////////////////////////////////////////////////////////////////////
// CListIterInt Class
//

CListIterInt::CListIterInt()
{
  m_pCListIter = NULL;
}

CListIterInt::~CListIterInt()
{
  Uninit();
}

BOOL CListIterInt::Init(CListInt *pCListInt)
{
  ASSERT( pCListInt != NULL );

  if ( m_pCListIter == NULL )
    {      
      m_pCListIter = new CListIter<VOID*, CKeyInt>;
      ASSERT( m_pCListIter != NULL );
    }

  return ((CListIter<VOID*, CKeyInt>*)m_pCListIter)->
           Init((CList<VOID*, CKeyInt>*)(pCListInt->m_pCList));
}

VOID CListIterInt::Uninit()
{
  if ( m_pCListIter != NULL )
    {
      delete m_pCListIter;
      m_pCListIter = NULL;
    }
}

VOID *CListIterInt::PeekForward(INT_32 iKey)
{
  CKeyInt key;

  ASSERT( m_pCListIter != NULL );

  key.Set(iKey);

  return ((CListIter<VOID*, CKeyInt>*)m_pCListIter)->PeekForward(key);
}

VOID *CListIterInt::PeekBackward(INT_32 iKey)
{
  CKeyInt key;

  ASSERT( m_pCListIter != NULL );

  key.Set(iKey);

  return ((CListIter<VOID*, CKeyInt>*)m_pCListIter)->PeekBackward(key);
}

VOID *CListIterInt::PeekFirst(INT_32 *piKey/* = NULL */)
{
  VOID *pvData;

  ASSERT( m_pCListIter != NULL );

  if ( piKey == NULL )
    {
      pvData = ((CListIter<VOID*, CKeyInt>*)m_pCListIter)->PeekFirst(NULL);
    }
  else
    {
      CKeyInt key;
      key.SetNull();
      pvData = ((CListIter<VOID*, CKeyInt>*)m_pCListIter)->PeekFirst(&key);
      *piKey = key.Get();
    }

  return pvData;
}

VOID *CListIterInt::PeekNext(INT_32 *piKey/* = NULL*/)
{
  VOID *pvData;

  ASSERT( m_pCListIter != NULL );

  if ( piKey == NULL )
    {
      pvData = ((CListIter<VOID*, CKeyInt>*)m_pCListIter)->PeekNext(NULL);
    }
  else
    {
      CKeyInt key;
      key.SetNull();
      pvData = ((CListIter<VOID*, CKeyInt>*)m_pCListIter)->PeekNext(&key);
      *piKey = key.Get();
    }

  return pvData;
}

VOID *CListIterInt::PeekLast(INT_32 *piKey/* = NULL */)
{
  VOID *pvData;

  ASSERT( m_pCListIter != NULL );

  if ( piKey == NULL )
    {
      pvData = ((CListIter<VOID*, CKeyInt>*)m_pCListIter)->PeekLast(NULL);
    }
  else
    {
      CKeyInt key;
      key.SetNull();
      pvData = ((CListIter<VOID*, CKeyInt>*)m_pCListIter)->PeekLast(&key);
      *piKey = key.Get();
    }

  return pvData;
}

VOID *CListIterInt::PeekPrev(INT_32 *piKey/* = NULL*/)
{
  VOID *pvData;

  ASSERT( m_pCListIter != NULL );

  if ( piKey == NULL )
    {
      pvData = ((CListIter<VOID*, CKeyInt>*)m_pCListIter)->PeekPrev(NULL);
    }
  else
    {
      CKeyInt key;
      key.SetNull();
      pvData = ((CListIter<VOID*, CKeyInt>*)m_pCListIter)->PeekPrev(&key);
      *piKey = key.Get();
    }

  return pvData;
}


















