#include "matrix.h"
#include "point.h"
#include "quater.h"

Matrix::Matrix()
{
  SetIdentity();
}

Matrix::~Matrix()
{
}

VOID Matrix::SetNull()
{
  memset(m_m, 0, sizeof(MATRIX4D));
}

VOID Matrix::SetIdentity()
{
  UINT_32 i;

  SetNull();

  for ( i = 0; i < 16; i += 5 )
    {
      m_m[i] = 1.0;
    }
}

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 5: You need to implement these!!!
// 

VOID Matrix::SetTranslate(const Vector &v)
{
  SetIdentity();
}

// This is useful with the Rodrigues Formula for general axis rotations
VOID Matrix::SetCross(const Vector &v)
{
  SetNull();
}

// Ideally use the Rodrigues Formula to form the general axis rotation
VOID Matrix::SetRotate(const Vector &v, DOUBLE dTheta)
{
  SetIdentity();
}  

// Change of Basis rotation, which is useful to creat the LookAt transform
// mGa_b these are b's basis vectors represented in a's coordinate system
VOID Matrix::SetRotate(const Vector &vEx, const Vector &vEy, const Vector &vEz)
{
  SetIdentity();
}
// 
// END  : ASSIGNMENT 5: You need to implement these!!!
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 5: Not necessary, but if you really like Quaternions ...
// 

// Make the general axis rotation matrix using the quaternion description
// you can do this instead of the Rodrigues formula, if you really want
VOID Matrix::SetRotate(const Quater &q)
{
   SetIdentity();
}
// 
// END  : ASSIGNMENT 5: Not necessary, but if you really like Quaternions ...
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 5: You need to implement these!!!
// 
VOID Matrix::SetRotateX(DOUBLE dTheta)
{
  SetIdentity();
}

VOID Matrix::SetRotateY(DOUBLE dTheta)
{
  SetIdentity();
}

// This is the same as our 2D rotation so I will leave it filled in for you
VOID Matrix::SetRotateZ(DOUBLE dTheta)
{
  SetIdentity();
  m_m[0] = (FLOAT)cos(dTheta);
  m_m[4] = (FLOAT)(-sin(dTheta));
  m_m[1] = (FLOAT)sin(dTheta);
  m_m[5] = (FLOAT)cos(dTheta);
}

// This is the same as our 2D rotation so I will leave it filled in for you
// I can't remember where this is used if at all in the program
VOID Matrix::SetRotateZ(FLOAT sint, FLOAT cost)
{
  SetIdentity();
  m_m[0] = cost;
  m_m[4] = -sint;
  m_m[1] = sint;
  m_m[5] = cost;
}

// Non Uniform Scaling
VOID Matrix::SetScale(const Vector &v)
{
  SetIdentity();
}

// Uniform Scaling
VOID Matrix::SetScale(const FLOAT f)
{
  SetIdentity();
}

VOID Matrix::SetShearXY(const Vector &v)
{
  ASSERT( v(2) != 0.0 );

  SetIdentity();
}

VOID Matrix::SetShearYZ(const Vector &v)
{
  ASSERT( v(0) != 0.0 );

  SetIdentity();
}

VOID Matrix::SetShearXZ(const Vector &v)
{
  ASSERT( v(1) != 0.0 );

  SetIdentity();
}
// 
// END  : ASSIGNMENT 5: You need to implement these!!!
//////////////////////////////////////////////////////////////////////////////

VOID Matrix::SetViewPort(const Vector &vVPMin, const Vector &vVPMax, 
			 const Vector &vWWMin, const Vector &vWWMax, 
			 FLOAT fDYDX/* = 1.0*/)
{
  Matrix mTransWW, mTransVP, mScaleNonuniform, mScale;
  Vector vVP, vWW, vScale, v;
  FLOAT fScale;

  vWW = (vWWMin + vWWMax) / 2.0;
  vWW[2] = 0.0;
  mTransWW.SetTranslate(-vWW);

  vScale.Set(1.0, fDYDX, 1.0);
  mScaleNonuniform.SetScale(vScale);

  vWW = vWWMax - vWWMin;
  vWW[1] = fDYDX*vWW[1];
  vWW[2] = 0.0;
  vVP = vVPMax - vVPMin;
  vVP[2] = 0.0;
  
  if ( vWW[0] != 0.0 && vWW[1] != 0.0 )
    {
      fScale = MIN(vVP[0]/vWW[0], vVP[1]/vWW[1]);
    }
  else if ( vWW[0] != 0.0 )
    {
      fScale = vVP[0]/vWW[0];
    }
  else if ( vWW[1] != 0.0 )
    {
      fScale = vVP[1]/vWW[1];
    }
  else
    {
      fScale = 1.0;
    }

  vScale.Set(fScale, fScale, fScale);
  mScale.SetScale(vScale);

  v = (vVPMin + vVPMax) / 2.0;
  v[2] = -1.0;
  mTransVP.SetTranslate(v);

  *this = mTransVP * mScale * mScaleNonuniform * mTransWW;
}

FLOAT Matrix::Trace() const
{
  return ( m_m[0] + m_m[5] + m_m[10] );
}

FLOAT Matrix::Determinant() const
{
  return  ( ( -m_m[ 3] * ( ( (m_m[ 4] * m_m[ 9] * m_m[14]) +
			     (m_m[ 8] * m_m[13] * m_m[ 6]) +
			     (m_m[12] * m_m[ 5] * m_m[10]) ) -
			   ( (m_m[ 6] * m_m[ 9] * m_m[12]) +
			     (m_m[10] * m_m[13] * m_m[ 4]) +
			     (m_m[14] * m_m[ 5] * m_m[ 8]) ) ) ) +
	    (  m_m[ 7] * ( ( (m_m[ 0] * m_m[ 9] * m_m[14]) +
			     (m_m[ 8] * m_m[13] * m_m[ 2]) +
			     (m_m[12] * m_m[ 1] * m_m[10]) ) -
			   ( (m_m[ 2] * m_m[ 9] * m_m[12]) +
			     (m_m[10] * m_m[13] * m_m[ 0]) +
			     (m_m[14] * m_m[ 1] * m_m[ 8]) ) ) ) +
	    ( -m_m[11] * ( ( (m_m[ 0] * m_m[ 5] * m_m[14]) +
			     (m_m[ 4] * m_m[13] * m_m[ 2]) +
			     (m_m[12] * m_m[ 1] * m_m[ 6]) ) -
			   ( (m_m[ 2] * m_m[ 5] * m_m[12]) +
			     (m_m[ 6] * m_m[13] * m_m[ 0]) +
			     (m_m[14] * m_m[ 1] * m_m[ 4]) ) ) ) +
	    (  m_m[15] * ( ( (m_m[ 0] * m_m[ 5] * m_m[10]) +
			     (m_m[ 4] * m_m[ 9] * m_m[ 2]) +
			     (m_m[ 8] * m_m[ 1] * m_m[ 6]) ) -
			   ( (m_m[ 2] * m_m[ 5] * m_m[ 8]) +
			     (m_m[ 6] * m_m[ 9] * m_m[ 0]) +
			     (m_m[10] * m_m[ 1] * m_m[ 4]) ) ) ) );
}

VOID Matrix::Transpose()
{
  UINT_32 i, j;
  FLOAT f;

  for ( i = 0; i < (4-1); i++ )
    {
      for ( j = i+1; j < 4; j++)
	{
	  f = m_m[Index(i,j)];
	  m_m[Index(i,j)] = m_m[Index(j,i)];
	  m_m[Index(j,i)] = f;
	}
    }
}

Matrix &Matrix::operator=(const Matrix& m)
{
  memcpy(m_m, m.m_m, sizeof(MATRIX4D));

  return (*this);
}

Matrix &Matrix::operator+=(const Matrix& m)
{
  UINT_32 i;

  for ( i = 0; i < 16; i++ )
    {
      m_m[i] = m_m[i] + m.m_m[i];
    }

  return (*this);
}

Matrix &Matrix::operator-=(const Matrix& m)
{
  UINT_32 i;

  for ( i = 0; i < 16; i++ )
    {
      m_m[i] = m_m[i] - m.m_m[i];
    }

  return (*this);
}

Matrix &Matrix::operator*=(const FLOAT f)
{
  UINT_32 i;

  for ( i = 0; i < 16; i++ )
    {
      m_m[i] = m_m[i] * f;
    }

  return (*this);
}

Matrix &Matrix::operator/=(const FLOAT f)
{
  UINT_32 i;

  ASSERT( f != 0.0 );

  for ( i = 0; i < 16; i++ )
    {
      m_m[i] = m_m[i] / f;
    }

  return (*this);
}

Matrix Matrix::operator*(const Matrix& m) const
{
  UINT_32 i, j, k;
  UINT_32 i4, k4;

  Matrix mResult;

  i4 = 0;
  for ( i = 0; i < 4; i++ ) // Col Selector
    {
      for ( j = 0; j < 4; j++ ) // Row Selector
	{
	  mResult.m_m[i4+j] = 0.0;
	  k4 = 0;
	  for ( k = 0; k < 4; k++ ) // Iterator
	    {
	      mResult.m_m[i4+j] += m_m[k4+j] * m.m_m[i4+k];
	      k4 += 4;
	    }
	}
      i4 += 4;
    }

  return mResult;
}

Vector Matrix::operator*(const Vector &v) const
{
  UINT_32 i, j, j4;
  Vector vResult;

  for ( i = 0; i < 4; i++ )
    {
      vResult[i] = 0.0f;
      j4 = 0;
      for ( j = 0; j < 4; j++ )
	{
	  vResult[i] += m_m[i + j4] * v(j);
	  j4 += 4;
	}
    }

  return vResult;
}

Vector operator*(const Vector &v, const Matrix &m)
{
  UINT_32 i, j, k;
  Vector vResult;
  MATRIX4D mTemp;

  m.Get(mTemp);

  k = 0;
  for ( i = 0; i < 4; i++ )
    {
      vResult[i] = 0.0f;
      for ( j = 0; j < 4; j++ )
	{
	  vResult[i] += mTemp[k] * v(j);
	  k++;
	}
    }

  return vResult;
}

Point Matrix::operator*(const Point &pt) const
{
  UINT_32 i, j, j4;
  Point ptResult;

  for ( i = 0; i < 4; i++ )
    {
      ptResult[i] = 0.0f;
      j4 = 0;
      for ( j = 0; j < 4; j++ )
	{
	  ptResult[i] += m_m[i + j4] * pt(j);
	  j4 += 4;
	}
    }

  return ptResult;
}

VOID Matrix::Display() const
{
  UINT_32 i, j;

  for ( i = 0; i < 4; i++ )
    {
      printf("| ");
      for ( j = 0; j < 4; j++ )
	{
	  printf("%1.3f ", m_m[Index(i,j)]);
	}
      printf("|\n");
    }
}




