#include "vertex.h"
#include "slideprimitive.h"

Vertex::Vertex()
{
  Null();
}

Vertex::Vertex(const Point &pt, const Vector &vNormal, const Vector &vColor,
	       const Point &ptTexture)
{
  m_pt = pt;
  m_vNormal = vNormal;
  m_vColor = vColor;
  m_ptTexture = ptTexture;
}

Vertex::~Vertex()
{
}

VOID Vertex::Null()
{
  m_pt.Null();
  m_vNormal.Null();
  m_vColor.Null();
  m_ptTexture.Null();
}

BOOL Vertex::Homogenize()
{
  BOOL bCorrect;

  bCorrect = m_pt.Homogenize();
  // We might want to do something about cutting off Color values as well

  return bCorrect;
}

Vertex &Vertex::operator=(const Vertex &vx)
{
  m_pt = vx.m_pt;
  m_vNormal = vx.m_vNormal;
  m_vColor = vx.m_vColor;
  m_ptTexture = vx.m_ptTexture;

  return *this;
}

Vertex &Vertex::operator+=(const Vertex &vx)
{
  m_pt += vx.m_pt;
  m_vNormal += vx.m_vNormal;
  m_vColor += vx.m_vColor;
  m_ptTexture += vx.m_ptTexture;

  return *this;
}

Vertex &Vertex::operator-=(const Vertex &vx)
{
  Vector v;

  v = m_pt - vx.m_pt;
  m_pt.Set(v[0], v[1], v[2], v[3]);
  m_vNormal -= vx.m_vNormal;
  m_vColor -= vx.m_vColor;
  v = m_ptTexture - vx.m_ptTexture;
  m_ptTexture.Set(v[0], v[1], v[2], v[3]);

  return *this;
}

Vertex &Vertex::operator*=(const FLOAT f)
{
  m_pt *= f;
  m_vNormal *= f;
  m_vColor *= f;
  m_ptTexture *= f;

  return *this;
}

Vertex &Vertex::operator/=(const FLOAT f)
{
  m_pt /= f;
  m_vNormal /= f;
  m_vColor /= f;
  m_ptTexture /= f;

  return *this;
}

VOID Vertex::Render()
{
  gilVertex3fv(m_pt.Get());
}
